#!/usr/bin/python
import json
import requests
from beanstalk_config import *
from beanstalk_utils import *
import argparse
import sys
import os

# Flush STOUT continuously
sys.stdout = os.fdopen(sys.stdout.fileno(), 'w', 0)

weather_data_server_tube = 'WeatherServerTube'

API_KEY = '9263a3d2085659767ddbad9544454a3b'
openweatherAPIPrefix = "http://api.openweathermap.org/data/2.5/find"

class CityMetadata:
    def __init__(self, cityName, serverName):
        self.name = cityName
        self.serverName = serverName
        self.queryString = openweatherAPIPrefix + "?q=%s&type=like&mode=json" % cityName
        
    def initWeatherDataURL(self):
        response = requests.get(self.queryString)
        if (response.status_code == 200):
            parsed_data = json.loads(response.text)
            self.cityRecord = City(parsed_data["list"][0])
            self.weatherReportURL = self.cityRecord.weatherReportUrl()
        else:
            self.weatherReportURL = None
            
class City:
    def __init__(self, citySpec):
        self.name = citySpec["name"]
        location = citySpec["coord"]
        self.lat = location["lat"]
        self.lon = location["lon"]
    
    def weatherReportUrl(self):
        return openweatherAPIPrefix + "?lat=%s&lon=%s&cnt=100" % (self.lat, self.lon)
    
    def __str__(self):
        return "%s: Lat: %s Lon: %s" % (self.name, self.lat, self.lon)
        
def kelvinToCelsius(temperature):
    return temperature - 272.15

def kelvinToFarenheit(temperature):
    celsius = kelvinToCelsius(temperature)
    return celsius * 1.8 + 32


class CityWeatherReport:
    def __init__(self, weatherReport):
        self.city = City(weatherReport)
        self.weather = weatherReport["weather"]
        self.clouds = weatherReport["clouds"]
        self.pressure = weatherReport["main"]["pressure"]
        self.humidity = weatherReport["main"]["humidity"]
        self.highTemp = weatherReport["main"]["temp_max"]
        self.lowTemp = weatherReport["main"]["temp_min"]
        self.temperature = weatherReport["main"]["temp"]
        self.wind = weatherReport["wind"]
        self.rain = None
        if "rain" in weatherReport:
            rainReport = weatherReport["rain"]
            self.rain = rainReport
        self.weatherDetail = weatherReport["weather"]
        
    def rainWarning(self):
        maxRain = 0
        if ('3h' in self.rain):
            maxRain = self.rain['3h']
        if ('1h' in self.rain):
            maxRain = max(maxRain, self.rain('1h'))
        if (maxRain > 1.0):
            return {"level": "Caution", "metric": "Rain", "value": maxRain}
        
    def severeWeather(self):
        result = []
        hI = self.heatIndex()
        hICautions = [(120, "Extreme Danger"), (105, "Danger"), (90, "Extreme Caution"), (80, "Caution")]
        actualCautions = [caution for caution in hICautions if caution[0] > hI]
        if (len(actualCautions) > 0):
            highestCaution = actualCautions[0]
            result.append({"level":highestCaution[0], "metric": "Heat Index", "value":highestCaution[1]})
        if self.rain:
            rainWarning = self.rainWarning()
            if rainWarning: result.append(rainWarning)
        return result
        
    
    def heatIndex(self):
        temperature = kelvinToFarenheit(self.highTemp)
        terms = [1, temperature, self.humidity, temperature * self.humidity, temperature * temperature, self.humidity * self.humidity,
                 temperature * temperature * self.humidity, temperature * self.humidity * self.humidity, temperature * temperature * self.humidity * self.humidity]
        constants = [-42.379, 2.04901523,10.14333127,-0.22475541,-6.83783E-3,-5.481717E-2, 1.22874E-3, 8.5282E-4,-1.99E-6]
        result = sum([constants[i] * terms[i] for i in range(0, len(constants))])
        return result
    
    def __str__(self):
        return "%s, weather %s, clouds %s, pressure %d, humidity %d, high %d, low %d, current %d, heat index %d, wind %s" % (str(self.city), str(self.weather), str(self.clouds), self.pressure, self.humidity, self.highTemp, self.lowTemp, self.temperature, self.heatIndex(), str(self.wind))
        
        

def server():
    cityQueries = [CityMetadata('Salt_Lake_City','pc24.utahddc.geniracks.net'), CityMetadata('Chicago','pc4.instageni.illinois.edu'),
                   CityMetadata('Washington_DC','pc5.instageni.maxgigapop.net')]
    for city in cityQueries:
        city.initWeatherDataURL()
    for tube in client_tubes: beanstalk.ignore(tube)
    outstandingQueries = 0
    for city in cityQueries:
        if city.weatherReportURL:
            beanstalk.use(city.serverName)
            beanstalk.put(city.weatherReportURL)
            outstandingQueries += 1

def client():
    job = beanstalk.reserve()
    urlToGet = job.body
    responseToGet = requests.get(urlToGet)
    if responseToGet.status_code != 200:
        print ("No response for %s" % urlToGet)
        exit(1)
    weatherData = json.loads(responseToGet.text)
    if not "list" in weatherData:
        print("No reports returned for %s " % urlToGet)
        exit(0)
    reports = weatherData["list"]
    weatherReports = [CityWeatherReport(report) for report in reports]
    print "Severe Weather Warnings"
    for cityWeatherReport in weatherReports:
        print "%s: %s" % (cityWeatherReport.city.name, cityWeatherReport.severeWeather())
        
parser = argparse.ArgumentParser()
parser.add_argument("-c", "--client", help="run as a job client (default, disables -s)", action="store_true")
parser.add_argument("-s", "--server", help="run as a job server", action="store_true")

args = parser.parse_args()
run_server = False # set default
if (args.server):
    run_server = True
if (args.client):
   run_server = False  # run with -c -s disables -s

if (run_server):
    server()
else:
    client()

    
    
    
            
            
            

